@@ -1,7 +1,7 @@
-require File.join(File.dirname(__FILE__), "test_helper")
-require File.join(File.dirname(__FILE__), '..', 'lib', 'timecop')
+require_relative "test_helper"
+require 'timecop'
 
-class TestTimecop < Minitest::Unit::TestCase
+class TestTimecop < Minitest::Test
   def teardown
     Timecop.return
   end
@@ -251,7 +251,7 @@ class TestTimecop < Minitest::Unit::TestCase
       new_now = Time.now
       assert_times_effectively_equal(new_now, t, 1, "Looks like we failed to actually travel time")
       sleep(0.25)
-      assert_times_effectively_not_equal new_now, Time.now, 0.25, "Looks like time is not moving"
+      assert_times_effectively_not_equal new_now, Time.now, 0.24, "Looks like time is not moving"
     end
   end
 
@@ -495,6 +495,18 @@ class TestTimecop < Minitest::Unit::TestCase
     end
   end
 
+  def test_raises_when_safe_mode_and_no_block_though_previously_block_given
+    Timecop.freeze do
+      Timecop.freeze
+    end
+
+    with_safe_mode do
+      assert_raises Timecop::SafeModeException do
+        Timecop.freeze
+      end
+    end
+  end
+
   def test_no_raise_when_safe_mode_and_block_used
     with_safe_mode do
       Timecop.freeze {}
@@ -507,6 +519,14 @@ class TestTimecop < Minitest::Unit::TestCase
     end
   end
 
+  def test_no_raise_when_safe_mode_and_no_block_and_in_block_context
+    with_safe_mode do
+      Timecop.freeze do
+        Timecop.freeze
+      end
+    end
+  end
+
   def test_date_strptime_without_year
     Timecop.freeze(Time.new(1984,2,28)) do
       assert_equal Date.strptime('04-14', '%m-%d'), Date.new(1984, 4, 14)
@@ -536,6 +556,23 @@ class TestTimecop < Minitest::Unit::TestCase
     assert !Timecop.frozen?
   end
 
+  def test_thread_safe_timecop
+    Timecop.thread_safe = true
+    date = Time.local(2011, 01, 02)
+    thread = Thread.new do
+      Timecop.freeze(date) do
+        sleep 1 #give main thread time to run
+        assert_equal date, Time.now
+      end
+    end
+
+    sleep 0.25
+    assert Time.now != date
+    thread.join
+  ensure
+    Timecop.thread_safe = false
+  end
+
   private
 
   def with_safe_mode(enabled=true)
